﻿using System;
using System.Collections.Generic;
using PRISMResourceTest.Infrastructure.Services;
using PRISMResourceTest.Infrastructure.Base;
using System.Resources;
using PRISMResourceTest.Infrastructure.Events;
using Microsoft.Practices.Composite.Events;
using System.Threading;
using System.Globalization;

namespace PRISMResourceTest.ApplicationCore.Services
{
    /// <summary>
    /// Backend service for managing ResourceManagers and their resources
    /// 
    /// Stores ResourceManagers as a WeakReference
    /// </summary>
    public class ResourceManagerService : IResourceManagerService
    {
        private IEventAggregator _eventAggregator;
        private Dictionary<string, WeakReference> _managers;

        /// <summary>
        /// Current application locale
        /// </summary>
        public Locale CurrentLocale { get; private set; }

        /// <summary>
        /// Initializes a new instance of the ResourceManager class.
        /// </summary>
        public ResourceManagerService(IEventAggregator eventAggregator)
        {
            _eventAggregator = eventAggregator;
            _managers = new Dictionary<string, WeakReference>();

            // Set to default culture
            ChangeLocale(CultureInfo.CurrentCulture.IetfLanguageTag);
        }

        /// <summary>
        /// Retreives a string resource with the given key from the given
        /// resource manager.
        /// 
        /// Will load the string relevant to the current culture.
        /// </summary>
        /// <exception cref="ArgumentException">Thrown if the manager does not exist or has been disposed</exception>
        /// <param name="managerName">Name of the ResourceManager</param>
        /// <param name="resourceKey">Resource to lookup</param>
        /// <returns></returns>
        public string GetResourceString(string managerName, string resourceKey)
        {
            WeakReference reference = null;
            ResourceManager manager = null;
            string resource = String.Empty;

            if (!_managers.TryGetValue(managerName, out reference))
                throw new ArgumentException("managerName must be a valid manager");

            manager = reference.Target as ResourceManager;
            if (manager == null)
            {
                UnregisterManager(managerName);
                throw new ArgumentException("managerName must be a valid manager");
            }

            resource = manager.GetString(resourceKey);

            return resource;
        }

        /// <summary>
        /// Changes the current locale
        /// </summary>
        /// <param name="newLocaleName">IETF locale name (e.g. en-US, en-GB)</param>
        public void ChangeLocale(string newLocaleName)
        {
            CultureInfo newCultureInfo = new CultureInfo(newLocaleName);
            Thread.CurrentThread.CurrentCulture = newCultureInfo;
            Thread.CurrentThread.CurrentUICulture = newCultureInfo;

            Locale newLocale = new Locale() { Name = newLocaleName, RTL = newCultureInfo.TextInfo.IsRightToLeft };
            CurrentLocale = newLocale;

            _eventAggregator.GetEvent<LocaleChangedEvent>().Publish(newLocale);
        }

        /// <summary>
        /// Fires the LocaleChange event to reload bindings
        /// </summary>
        public void Refresh()
        {
            ChangeLocale(CultureInfo.CurrentCulture.IetfLanguageTag);
        }

        /// <summary>
        /// Register a ResourceManager, does not fire a refresh
        /// </summary>
        /// <param name="managerName">Name to store the manager under, used with GetResourceString/UnregisterManager</param>
        /// <param name="manager">ResourceManager to store</param>
        public void RegisterManager(string managerName, ResourceManager manager)
        {
            RegisterManager(managerName, manager, false);
        }

        /// <summary>
        /// Register a ResourceManager
        /// </summary>
        /// <param name="managerName">Name to store the manager under, used with GetResourceString/UnregisterManager</param>
        /// <param name="manager">ResourceManager to store</param>
        /// <param name="refresh">Whether to fire the LocaleChanged event to refresh bindings</param>
        public void RegisterManager(string managerName, ResourceManager manager, bool refresh)
        {
            WeakReference _reference = null;

            _managers.TryGetValue(managerName, out _reference);

            if (_reference == null)
                _managers.Add(managerName, new WeakReference(manager));
            else if (_reference.Target == null)
                _managers[managerName] = new WeakReference(manager);

            if (refresh)
                Refresh();
        }

        /// <summary>
        /// Remove a ResourceManager
        /// </summary>
        /// <param name="name">Name of the manager to remove</param>
        public void UnregisterManager(string name)
        {
            WeakReference _reference = null;

            _managers.TryGetValue(name, out _reference);

            if (_reference != null)
                _managers.Remove(name);
        }
    }
}
